@@ -7,16 +7,19 @@
 
 this_file = __file__.rsplit('.', 1)[0]
 if this_file.endswith('caja-thg'):
-    from gi.repository import Caja as Nautilus
+    from gi.repository import Caja as Nautilus  # pytype: disable=import-error
     idstr_prefix = 'HgCaja'
 else:
-    from gi.repository import Nautilus
+    from gi.repository import Nautilus  # pytype: disable=import-error
     idstr_prefix = 'HgNautilus3'
 
-from gi.repository import GObject
-from gi.repository import GLib
-from gi.repository import Gio
-from gi.repository import Gtk
+from gi.repository import (  # pytype: disable=import-error
+    GLib,
+    GObject,
+    Gio,
+    Gtk,
+)
+
 import os
 import sys
 
@@ -52,7 +55,19 @@ def _thg_path():
         sys.path.insert(0, thgpath)
 _thg_path()
 
-from tortoisehg.util import hglib, paths, debugthg, cachethg
+from tortoisehg.util import hglib, menuthg, paths, debugthg, cachethg
+
+if hglib.TYPE_CHECKING:
+    from typing import (
+        Any,
+        Dict,
+        List,
+        Optional,
+        Text,
+    )
+    from mercurial import (
+        localrepo,
+    )
 
 if debugthg.debug('N'):
     debugf = debugthg.debugf
@@ -68,10 +83,9 @@ class HgExtensionDefault(GObject.GObject):
 
     def __init__(self):
         self.scanStack = []
-        self.allvfs = {}
+        self.allvfs = {}  # type: Dict[Text, Any]
         self.inv_dirs = set()
 
-        from tortoisehg.util import menuthg
         self.hgtk = paths.find_in_path(thg_main)
         self.menu = menuthg.menuThg()
 
@@ -97,11 +111,14 @@ class HgExtensionDefault(GObject.GObject):
         self.gmon.connect('changed', self.notified)
 
     def icon(self, iname):
+        # type: (Text) -> Optional[Text]
         return paths.get_tortoise_icon(iname)
 
     def get_path_for_vfs_file(self, vfs_file, store=True):
+        # type: (Any, bool) -> Optional[Text]
         if vfs_file.get_uri_scheme() != 'file':
             return None
+        # TODO: is get_uri returing unicode?
         path = hglib.tounicode(util.urlreq.unquote(vfs_file.get_uri()[7:]))
         if vfs_file.is_gone():
             self.allvfs.pop(path, '')
@@ -111,6 +128,7 @@ class HgExtensionDefault(GObject.GObject):
         return path
 
     def get_vfs(self, path):
+        # type: (Text) -> Any
         vfs = self.allvfs.get(path, None)
         if vfs and vfs.is_gone():
             del self.allvfs[path]
@@ -118,6 +136,7 @@ class HgExtensionDefault(GObject.GObject):
         return vfs
 
     def get_repo_for_path(self, path):
+        # type: (Text) -> Optional[localrepo.localrepository]
         '''
         Find mercurial repository for vfs_file
         Returns hg.repo
@@ -134,6 +153,7 @@ class HgExtensionDefault(GObject.GObject):
             return None
 
     def run_dialog(self, menuitem, hgtkcmd, cwd = None, files = None):
+        # type: (Any, Text, Optional[Text], List[Text]) -> None
         '''
         hgtkcmd - hgtk subcommand
         '''
@@ -181,6 +201,7 @@ class HgExtensionDefault(GObject.GObject):
         return self._buildMenu(menus, files)
 
     def _buildMenu(self, menus, files):
+        # type: (menuthg.thg_menu, List[Text]) -> List[Nautilus.MenuItem]
         '''Build one level of a menu'''
         items = []
         if files:
@@ -195,6 +216,7 @@ class HgExtensionDefault(GObject.GObject):
                 # can not insert a separator till now
                 pass
             elif menu_info.isSubmenu():
+                assert isinstance(menu_info, menuthg.TortoiseSubmenu)  # help pytype
                 if hasattr(Nautilus, 'Menu'):
                     item = Nautilus.MenuItem(name=idstr, label=menu_info.menutext,
                             tip=menu_info.helptext)
@@ -207,6 +229,8 @@ class HgExtensionDefault(GObject.GObject):
                     for subitem in self._buildMenu(menu_info.get_menus(), files):
                         items.append(subitem)
             else:
+                assert isinstance(menu_info,
+                                  menuthg.TortoiseMenu)  # help pytype
                 if menu_info.state:
                     item = Nautilus.MenuItem.new(idstr,
                                  menu_info.menutext,
@@ -234,6 +258,7 @@ class HgExtensionDefault(GObject.GObject):
                                "Version control status"),
 
     def _get_file_status(self, localpath, repo=None):
+        # type: (Text, Optional[localrepo.localrepository]) -> Any
         cachestate = cachethg.get_state(localpath, repo)
         cache2state = {cachethg.UNCHANGED:   ('default',   'clean'),
                        cachethg.ADDED:       ('list-add',  'added'),
@@ -266,6 +291,7 @@ class HgExtensionDefault(GObject.GObject):
             self.invalidate(files, root)
 
     def invalidate(self, paths, root = ''):
+        # type: (List[Text], Text) -> None
         started = bool(self.inv_dirs)
         if cachethg.cache_pdir == root and root not in self.inv_dirs:
             cachethg.overlay_cache.clear()
@@ -294,6 +320,7 @@ class HgExtensionDefault(GObject.GObject):
 
     # property page borrowed from http://www.gnome.org/~gpoo/hg/nautilus-hg/
     def __add_row(self, row, label_item, label_value):
+        # type: (Any, Text, Text) -> None
         label = Gtk.Label(label_item)
         label.set_use_markup(True)
         label.set_alignment(1, 0)
@@ -327,14 +354,12 @@ class HgExtensionDefault(GObject.GObject):
         except:
             rev = ctx.rev()
         ctx = repo[rev]
-        node = short(ctx.node())
-        date = dateutil.datestr(ctx.date(), '%Y-%m-%d %H:%M:%S %1%2')
-        parents = '\n'.join([short(p.node()) for p in ctx.parents()])
-        description = ctx.description()
-        user = ctx.user()
-        user = GLib.markup_escape_text(user)
-        tags = b', '.join(ctx.tags())
-        branch = ctx.branch()
+        date = hglib.tounicode(dateutil.datestr(ctx.date(),
+                                                b'%Y-%m-%d %H:%M:%S %1%2'))
+        description = hglib.tounicode(ctx.description())
+        user = GLib.markup_escape_text(hglib.tounicode(ctx.user()))
+        tags = hglib.tounicode(b', '.join(ctx.tags()))
+        branch = hglib.tounicode(ctx.branch())
 
         self.property_label = Gtk.Label('Mercurial')
 
