@@ -27,7 +27,6 @@ from .qtgui import (
 from mercurial import (
     error,
     pycompat,
-    scmutil,
 )
 
 from ..util import hglib
@@ -159,7 +158,7 @@ class BisectDialog(QDialog):
 
     def _lookupRevision(self, changeid):
         try:
-            ctx = scmutil.revsymbol(self.repo, hglib.fromunicode(changeid))
+            ctx = hglib.revsymbol(self.repo, hglib.fromunicode(changeid))
             return ctx.rev()
         except (error.LookupError, error.RepoLookupError) as e:
             self._stbar.showMessage(hglib.tounicode(str(e)))
