@@ -52,11 +52,11 @@ from .qtgui import (
 )
 
 from mercurial import (
+    cmdutil,
     error,
     obsolete,  # delete if obsolete becomes enabled by default
     phases,
     pycompat,
-    scmutil,
 )
 from mercurial.utils import (
     dateutil,
@@ -84,6 +84,28 @@ from . import (
 )
 from .messageentry import MessageEntry
 
+if hglib.TYPE_CHECKING:
+    from typing import (
+        Callable,
+        Dict,
+        Iterable,
+        List,
+        Optional,
+        Text,
+        Tuple,
+        Union,
+    )
+    from mercurial import localrepo
+    from .qtcore import QObject
+    from .thgrepo import RepoAgent
+
+    # `opts` arg type internal to this module.  Public classes use
+    # Dict[Text, bytes] to support @command.
+    CommitOpts = Dict[Text, Union[bool, Text]]
+
+    # TODO: simplify to just be List[Text]?
+    CmdLines = List[List[Union[bytes, Text]]]
+
 if os.name == 'nt':
     from ..util import bugtraq
     _hasbugtraq = True
@@ -91,6 +113,7 @@ else:
     _hasbugtraq = False
 
 def readopts(repoagent):
+    # type: (RepoAgent) -> CommitOpts
     opts = {
         'ciexclude': repoagent.configString('tortoisehg', 'ciexclude'),
         'pushafter': repoagent.configString('tortoisehg', 'cipushafter'),
@@ -109,6 +132,7 @@ def readopts(repoagent):
     return opts
 
 def commitopts2str(opts, mode='commit'):
+    # type: (CommitOpts, Text) -> Text
     optslist = []
     for opt, value in opts.items():
         if opt in ['user', 'date', 'pushafter', 'autoinc',
@@ -123,9 +147,13 @@ def commitopts2str(opts, mode='commit'):
     return ' '.join(optslist)
 
 def mergecommitmessage(repo):
+    # type: (localrepo.localrepository) -> Text
     wctx = repo[None]
     engmsg = repo.ui.configbool(b'tortoisehg', b'engmsg')
-    if wctx.p1().branch() == wctx.p2().branch():
+    bcustommsg = repo.ui.config(b'tortoisehg', b'mergecommitmessage')
+    if bcustommsg:
+        text = hglib.tounicode(cmdutil.rendertemplate(wctx, bcustommsg))
+    elif wctx.p1().branch() == wctx.p2().branch():
         msgset = i18n.keepgettext()._('Merge')
         text = engmsg and msgset['id'] or msgset['str']
         text = pycompat.unicode(text)
@@ -135,7 +163,8 @@ def mergecommitmessage(repo):
         text = pycompat.unicode(text) % hglib.tounicode(wctx.p2().branch())
     return text
 
-def _getUserOptions(opts, *optionlist):
+def _getUserOptions(opts, optionlist):
+    # type: (CommitOpts, Iterable[Text]) -> List[Text]
     out = []
     for opt in optionlist:
         if opt not in opts:
@@ -147,10 +176,15 @@ def _getUserOptions(opts, *optionlist):
             out.append('--' + opt)
         else:
             out.append('--' + opt)
-            out.append(val)
+            assert isinstance(val, str)  # help pytype
+            # Even with the assertion, the cast is needed or pytype thinks `out`
+            # is List[Union[bool, str]]
+            out.append(str(val))
     return out
 
 def _mqNewRefreshCommand(repo, isnew, stwidget, pnwidget, message, opts, olist):
+    # type: (localrepo.localrepository, bool, status.StatusWidget, QLineEdit, Text, CommitOpts, Iterable[Text]) -> Optional[CmdLines]
+    # TODO: convert all bytes to unicode in the return value
     if isnew:
         name = pnwidget.text()
         if not name:
@@ -163,7 +197,7 @@ def _mqNewRefreshCommand(repo, isnew, stwidget, pnwidget, message, opts, olist):
         cmdline = ['qrefresh']
     if message:
         cmdline += ['--message=' + hglib.tounicode(message)]
-    cmdline += _getUserOptions(opts, *olist)
+    cmdline += _getUserOptions(opts, olist)
     files = ['--'] + [repo.wjoin(x) for x in stwidget.getChecked()]
     addrem = [repo.wjoin(x) for x in stwidget.getChecked('!?')]
     if len(files) > 1:
@@ -202,6 +236,9 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
     progress = pyqtSignal(str, object, str, str, object)
 
     def __init__(self, repoagent, pats, opts, parent=None, rev=None):
+        # type: (RepoAgent, List[bytes], Dict[Text, bytes], Optional[QObject], Optional[Text]) -> None
+        # TODO: drop `opts`, since it is immediately overwritten?
+        # TODO: make `rev` a byte string?
         QWidget.__init__(self, parent)
 
         repoagent.configChanged.connect(self.refresh)
@@ -321,8 +358,7 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
             self.pnlabel = QLabel()
             pnhbox.addWidget(self.pnlabel)
             self.pnedit = QLineEdit()
-            if hasattr(self.pnedit, 'setPlaceholderText'):  # Qt >= 4.7
-                self.pnedit.setPlaceholderText(_('### patch name ###'))
+            self.pnedit.setPlaceholderText(_('### patch name ###'))
             self.pnedit.setMaximumWidth(250)
             pnhbox.addWidget(self.pnedit)
             pnhbox.addStretch()
@@ -369,14 +405,17 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
 
     @property
     def repo(self):
+        # type: () -> localrepo.localrepository
         return self._repoagent.rawRepo()
 
     @property
     def rev(self):
+        # type: () -> Text
         """Return current revision"""
         return self._rev
 
     def selectRev(self, rev):
+        # type: (Text) -> None
         """
         Select the revision that must be set when the dialog is shown again
         """
@@ -385,6 +424,7 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
     @pyqtSlot(int)
     @pyqtSlot(object)
     def setRev(self, rev):
+        # TODO: is int even possible here?
         """Change revision to show"""
         self.selectRev(rev)
         if self.hasmqbutton:
@@ -395,6 +435,7 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
                     actionName=preferredActionName)
 
     def _getPreferredActionName(self):
+        # type: () -> Text
         """Select the preferred action, depending on the selected revision"""
         if not self.hasmqbutton:
             return 'commit'
@@ -412,9 +453,11 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
                 return 'qref'
 
     def commitSetupButton(self):
-        ispatch = lambda r: b'qtip' in r[b'.'].tags()
-        notpatch = lambda r: b'qtip' not in r[b'.'].tags()
+        # type: () -> QToolButton
+        ispatch = lambda r: b'qtip' in r[b'.'].tags()  # type: Callable[[localrepo.localrepository], bool]
+        notpatch = lambda r: b'qtip' not in r[b'.'].tags()  # type: Callable[[localrepo.localrepository], bool]
         def canamend(r):
+            # type: (localrepo.localrepository) -> bool
             if ispatch(r):
                 return False
             ctx = r[b'.']
@@ -466,6 +509,7 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
                 self.repo = repo
                 QMenu.__init__(self, parent)
             def getActionByName(self, act):
+                # type: (Text) -> QAction
                 return [a for a in self.actions() if a._name == act][0]
             def showEvent(self, event):
                 for a in self.actions():
@@ -495,6 +539,8 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
 
     @pyqtSlot(bool)
     def commitSetAction(self, refresh=False, actionName=None):
+        # type: (bool, Optional[Text]) -> None
+        # TODO: should this slot also be declared with str arg?
         allowcs = False
         if actionName:
             selectedAction = \
@@ -567,6 +613,7 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
         self.lastAction = curraction._name
 
     def getBranchCommandLine(self):
+        # type: () -> Tuple[Optional[CmdLines], bool]
         '''
         Create the command line to change or create the selected branch unless
         it is the selected branch
@@ -588,7 +635,7 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
                 resp = 0
             else:
                 # TODO: should look up only in branch namespace
-                rev = scmutil.revsymbol(repo, branch).rev()
+                rev = hglib.revsymbol(repo, branch).rev()
                 resp = qtlib.CustomPrompt(_('Confirm Branch Change'),
                     _('Named branch "%s" already exists, '
                       'last used in revision %d\n'
@@ -607,6 +654,7 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
                  _('Cancel')), 2, 2).run()
         if resp == 0:
             newbranch = True
+            assert branch is not None
             commandlines.append(['branch', '--force', branch])
         elif resp == 2:
             return None, False
@@ -614,6 +662,8 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
 
     @pyqtSlot()
     def mqPerformAction(self):
+        # type: () -> Optional[bool]
+        # TODO: drop bool return?
         curraction = self.mqgroup.checkedAction()
         if curraction._name == 'commit':
             return self.commit()
@@ -641,6 +691,7 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
 
     @pyqtSlot(str, str)
     def fileDisplayed(self, wfile, contents):
+        # type: (Text, Text) -> None
         'Status widget is displaying a new file'
         if not (wfile and contents):
             return
@@ -685,10 +736,12 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
         self._apis.prepare()
 
     def apiPrepFinished(self):
+        # type: () -> None
         'QsciAPIs has finished parsing displayed file'
         self.msgte.lexer().setAPIs(self._apis)
 
     def bugTrackerPostCommit(self):
+        # type: () -> None
         if not _hasbugtraq or self.opts['bugtraqtrigger'] != 'commit':
             return
         # commit already happened, get last message in history
@@ -700,17 +753,21 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
         self.bugtraq = self.createBugTracker()
 
     def createBugTracker(self):
-        bugtraqid = self.opts['bugtraqplugin'].split(' ', 1)[0]
+        opt = self.opts['bugtraqplugin']
+        assert isinstance(opt, str)  # help pytype
+        bugtraqid = opt.split(' ', 1)[0]
         result = bugtraq.BugTraq(bugtraqid)
         return result
 
     def getBugTrackerCommitMessage(self):
+        # type: () -> None
         parameters = self.opts['bugtraqparameters']
         message = self.getMessage(True)
         newMessage = self.bugtraq.get_commit_message(parameters, message)
         self.setMessage(newMessage)
 
     def details(self):
+        # type: () -> None
         mode = 'commit'
         if len(self.repo[None].parents()) > 1:
             mode = 'merge'
@@ -725,12 +782,14 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
 
     @pyqtSlot(int)
     def repositoryChanged(self, flags):
+        # type: (int) -> None
         if flags & thgrepo.WorkingParentChanged:
             self._refreshWorkingState()
         elif flags & thgrepo.WorkingBranchChanged:
             self.refresh()
 
     def _refreshWorkingState(self):
+        # type: () -> None
         curraction = self.mqgroup.checkedAction()
         if curraction._name == 'commit' and not self.msgte.isModified():
             # default merge or close-branch message is outdated if new commit
@@ -755,11 +814,13 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
 
     @pyqtSlot()
     def refreshWctx(self):
+        # type: () -> None
         'User has requested a working context refresh'
         self.stwidget.refreshWctx() # Trigger reload of working context
 
     @pyqtSlot()
     def reload(self):
+        # type: () -> None
         'User has requested a reload'
         self.repo.thginvalidate()
         self.refresh()
@@ -767,6 +828,7 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
 
     @pyqtSlot()
     def refresh(self):
+        # type: () -> None
         ispatch = self.repo[b'.'].thgmqappliedpatch()
         if not self.hasmqbutton:
             self.commitButtonEnable.emit(not ispatch)
@@ -821,6 +883,7 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
             self.commitSetAction()
 
     def branchOp(self):
+        # type: () -> None
         d = branchop.BranchOpDialog(self._repoagent, self.branchop, self)
         d.setWindowFlags(Qt.Sheet)
         d.setWindowModality(Qt.WindowModal)
@@ -838,6 +901,7 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
             self.refresh()
 
     def topicOp(self):
+        # type: () -> None
         d = topic.TopicDialog(self._repoagent, self.branchop, self)
         d.setWindowFlags(Qt.Sheet)
         d.setWindowModality(Qt.WindowModal)
@@ -846,6 +910,7 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
         self.refresh()
 
     def canUndo(self):
+        # type: () -> Optional[Text]
         'Returns undo description or None if not valid'
         desc, oldlen = hglib.readundodesc(self.repo)
         if desc == 'commit':
@@ -853,6 +918,7 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
         return None
 
     def rollback(self):
+        # type: () -> None
         msg = self.canUndo()
         if not msg:
             return
@@ -863,6 +929,7 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
         self._runCommand('rollback', [['rollback']])
 
     def updateRecentMessages(self):
+        # type: () -> None
         # Define a menu that lists recent messages
         m = self.recentMessagesButton.menu()
         m.clear()
@@ -872,6 +939,7 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
             a.setData(s)
 
     def getMessage(self, allowreplace):
+        # type: (bool) -> bytes
         text = self.msgte.text()
         try:
             return hglib.fromunicode(text, 'strict')
@@ -883,6 +951,7 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
 
     @pyqtSlot(QAction)
     def msgSelected(self, action):
+        # type: (QAction) -> None
         if self.msgte.text() and self.msgte.isModified():
             d = QMessageBox.question(self, _('Confirm Discard Message'),
                         _('Discard current commit message?'),
@@ -894,16 +963,19 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
         self.msgte.setFocus()
 
     def setMessage(self, msg, modified=False):
+        # type: (Text, bool) -> None
         self.msgte.setText(msg)
         self.msgte.moveCursorToEnd()
         self.msgte.setModified(modified)
 
     def canExit(self):
+        # type: () -> bool
         if not self.stwidget.canExit():
             return False
         return self._cmdsession.isFinished()
 
     def loadSettings(self, s, prefix):
+        # type: (QSettings, Text) -> None
         'Load history, etc, from QSettings instance'
         repoid = hglib.shortrepoid(self.repo)
         lpref = prefix + '/commit/' # local settings (splitter, etc)
@@ -930,6 +1002,7 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
             pass
 
     def saveSettings(self, s, prefix):
+        # type: (QSettings, Text) -> None
         'Save history, etc, in QSettings instance'
         try:
             repoid = hglib.shortrepoid(self.repo)
@@ -946,6 +1019,7 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
             pass
 
     def addMessageToHistory(self, umsg):
+        # type: (Text) -> None
         umsg = pycompat.unicode(umsg)
         if umsg in self.msghistory:
             self.msghistory.remove(umsg)
@@ -954,13 +1028,15 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
         self.updateRecentMessages()
 
     def addUsernameToHistory(self, user):
-        user = hglib.tounicode(user)
+        # type: (Text) -> None
         if user in self.userhist:
             self.userhist.remove(user)
         self.userhist.insert(0, user)
         self.userhist = self.userhist[:10]
 
     def commit(self, amend=False):
+        # type: (bool) -> Optional[bool]
+        # TODO: drop the bool return?
         repo = self.repo
         try:
             msg = self.getMessage(False)
@@ -1142,7 +1218,9 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
                                         pycompat.maplist(hglib.tounicode,
                                                          self.files)))
         if len(repo[None].parents()) == 1:
-            for fname in self.opts.get('autoinc', '').split(','):
+            opt = self.opts.get('autoinc', '')
+            assert isinstance(opt, str)  # help pytype
+            for fname in opt.split(','):
                 fname = fname.strip()
                 if fname:
                     cmdline.append('glob:%s' % fname)
@@ -1160,6 +1238,7 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
         self._cmdsession.abort()
 
     def _runCommand(self, action, cmdlines):
+        # type: (Text, CmdLines) -> None
         self.currentAction = action
         self.progress.emit(*cmdui.startProgress(_topicmap[action], ''))
         self.commitButtonEnable.emit(False)
@@ -1169,6 +1248,7 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
         sess.commandFinished.connect(self.commandFinished)
 
     def commandFinished(self, ret):
+        # type: (int) -> None
         self.progress.emit(*cmdui.stopProgress(_topicmap[self.currentAction]))
         self.stopAction.setEnabled(False)
         self.commitButtonEnable.emit(True)
@@ -1201,6 +1281,7 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
 class DetailsDialog(QDialog):
     'Utility dialog for configuring uncommon settings'
     def __init__(self, repoagent, opts, userhistory, parent, mode='commit'):
+        # type: (RepoAgent, CommitOpts, List[Text], QWidget, Text) -> None
         QDialog.__init__(self, parent)
         self.setWindowTitle(_('%s - commit options') % repoagent.displayName())
         self._repoagent = repoagent
@@ -1222,6 +1303,7 @@ class DetailsDialog(QDialog):
         l = []
         v = opts.get('user')
         if v:
+            # TODO: should already be Text
             val = hglib.tounicode(v)
             self.usercb.setChecked(True)
             l.append(val)
@@ -1350,6 +1432,7 @@ class DetailsDialog(QDialog):
 
     @property
     def repo(self):
+        # type: () -> localrepo.localrepository
         return self._repoagent.rawRepo()
 
     def saveInRepo(self):
@@ -1360,6 +1443,7 @@ class DetailsDialog(QDialog):
         self.saveToPath(hglib.userrcpath())
 
     def saveToPath(self, path):
+        # type: (List[bytes]) -> None
         fn, cfg = hgrcutil.loadIniFile(path, self)
         if not hasattr(cfg, 'write'):
             qtlib.WarningMsgBox(_('Unable to save username'),
@@ -1451,9 +1535,10 @@ class DetailsDialog(QDialog):
                                 hglib.tounicode(str(e)), parent=self)
 
     def accept(self):
-        outopts = {}
+        outopts = {}  # type: Dict[Text, Text]
         if self.datecb.isChecked():
-            date = hglib.fromunicode(self.datele.text())
+            udate = self.datele.text()
+            date = hglib.fromunicode(udate)
             try:
                 dateutil.parsedate(date)
             except error.ParseError as e:
@@ -1464,12 +1549,12 @@ class DetailsDialog(QDialog):
                     err = hglib.tounicode(str(e))
                 qtlib.WarningMsgBox(_('Invalid date format'), err, parent=self)
                 return
-            outopts['date'] = date
+            outopts['date'] = udate
         else:
             outopts['date'] = ''
 
         if self.usercb.isChecked():
-            user = hglib.fromunicode(self.usercombo.currentText())
+            user = self.usercombo.currentText()
         else:
             user = ''
         outopts['user'] = user
@@ -1487,13 +1572,13 @@ class DetailsDialog(QDialog):
                 return
 
         if self.pushaftercb.isChecked():
-            remote = hglib.fromunicode(self.pushafterle.text())
+            remote = self.pushafterle.text()
             outopts['pushafter'] = remote
         else:
             outopts['pushafter'] = ''
 
         if self.autoinccb.isChecked():
-            outopts['autoinc'] = hglib.fromunicode(self.autoincle.text())
+            outopts['autoinc'] = self.autoincle.text()
         else:
             outopts['autoinc'] = ''
 
@@ -1510,6 +1595,7 @@ class CommitDialog(QDialog):
     'Standalone commit tool, a wrapper for CommitWidget'
 
     def __init__(self, repoagent, pats, opts, parent=None):
+        # type: (RepoAgent, List[bytes], Dict[Text, bytes], Optional[QObject]) -> None
         QDialog.__init__(self, parent)
         self.setWindowFlags(Qt.Window)
         self.setWindowIcon(qtlib.geticon('hg-commit'))
@@ -1565,16 +1651,19 @@ class CommitDialog(QDialog):
         qtlib.newshortcutsforstdkey(QKeySequence.Refresh, self, self.refresh)
 
     def linkActivated(self, link):
+        # type: (Text) -> None
         link = pycompat.unicode(link)
         if link.startswith('repo:'):
             self._subdialogs.open(link[len('repo:'):])
 
     def _createSubDialog(self, uroot):
+        # type: (Text) -> CommitDialog
         repoagent = self._repoagent.subRepoAgent(uroot)
         return CommitDialog(repoagent, [], {}, parent=self)
 
     @pyqtSlot()
     def updateUndo(self):
+        # type: () -> None
         BB = QDialogButtonBox
         undomsg = self.commit.canUndo()
         if undomsg:
@@ -1585,10 +1674,12 @@ class CommitDialog(QDialog):
             self.bb.button(BB.Discard).setToolTip('')
 
     def refresh(self):
+        # type: () -> None
         self.updateUndo()
         self.commit.reload()
 
     def postcommit(self):
+        # type: () -> None
         repo = self.commit.stwidget.repo
         if repo.ui.configbool(b'tortoisehg', b'closeci'):
             if self.commit.canExit():
@@ -1598,6 +1689,7 @@ class CommitDialog(QDialog):
                 QTimer.singleShot(0, self.reject)
 
     def promptExit(self):
+        # type: () -> bool
         exit = self.commit.canExit()
         if not exit:
             exit = qtlib.QuestionMsgBox(_('TortoiseHg Commit'),
@@ -1610,8 +1702,10 @@ class CommitDialog(QDialog):
         return exit
 
     def accept(self):
+        # type: () -> None
         self.commit.commit()
 
     def reject(self):
+        # type: () -> None
         if self.promptExit():
             QDialog.reject(self)
