@@ -149,7 +149,7 @@ class SummaryInfo(object):
             elif item == 'revid':
                 return str(ctx)
             elif item == 'gitcommit':
-                return hglib.gitcommit(ctx)
+                return hglib.gitcommit_short(ctx)
             elif item == 'desc':
                 return hglib.tounicode(ctx.description().replace(b'\0', b''))
             elif item == 'summary':
@@ -186,7 +186,7 @@ class SummaryInfo(object):
             elif item == 'branch':
                 value = self.get_data('rawbranch', *args)
                 if value:
-                    repo = ctx._repo
+                    repo = ctx.repo()
                     try:
                         if ctx.node() != repo.branchtip(ctx.branch()):
                             return None
@@ -260,7 +260,7 @@ class SummaryInfo(object):
                 p1 = ctx.p1()
                 if p1 is not None and p1.hex() == target:
                     return None
-                if target not in ctx._repo:
+                if target not in ctx.repo():
                     return None
                 return hglib.tounicode(target)
             raise UnknownItem(item)
