@@ -82,7 +82,7 @@ class _AbstractFileData(object):
         self.elabel = u''
         self.changes = None
 
-        self._textencoding = fileencoding.contentencoding(ctx._repo.ui)
+        self._textencoding = fileencoding.contentencoding(ctx.repo().ui)
 
     def createRebased(self, pctx):
         # new status is not known
@@ -152,13 +152,13 @@ class _AbstractFileData(object):
 
     def absoluteFilePath(self):
         """Absolute file-system path of this file"""
-        repo = self._ctx._repo
+        repo = self._ctx.repo()
         return hglib.tounicode(os.path.normpath(repo.wjoin(self._wfile)))
 
     def absoluteRepoRootPath(self):
         """Absolute file-system path to the root of the container repository"""
         # repo.root should be normalized
-        repo = self._ctx._repo
+        repo = self._ctx.repo()
         return hglib.tounicode(repo.root)
 
     def canonicalFilePath(self):
@@ -193,7 +193,7 @@ class _AbstractFileData(object):
         self._textencoding = fileencoding.canonname(name)
 
     def detectTextEncoding(self):
-        ui = self._ctx._repo.ui
+        ui = self._ctx.repo().ui
         # use file content for better guess; diff may be mixed encoding or
         # have immature multi-byte sequence
         data = self.contents or self.diff or b''
@@ -278,14 +278,14 @@ class FileData(_AbstractFileData):
             return None
 
         isbfile = False
-        repo = ctx._repo
+        repo = ctx.repo()
         maxdiff = repo.maxdiff
         self.flabel = u'<b>%s</b>' % self.filePath()
 
         if ctx2:
             # If a revision to compare to was provided, we must put it in
             # the context of the subrepo as well
-            if ctx2._repo.root != ctx._repo.root:
+            if ctx2.repo().root != ctx.repo().root:
                 wsub2, wfileinsub2, sctx2 = \
                     hglib.getDeepestSubrepoContainingFile(wfile, ctx2)
                 if wsub2:
@@ -472,7 +472,7 @@ class DirData(_AbstractFileData):
             return
 
         if not force:
-            self.error = _checkdifferror(self.diff, ctx._repo.maxdiff)
+            self.error = _checkdifferror(self.diff, ctx.repo().maxdiff)
             if self.error:
                 self.error += u'\n\n' + forcedisplaymsg
 
@@ -485,7 +485,7 @@ class PatchFileData(_AbstractFileData):
     def load(self, changeselect=False, force=False):
         ctx = self._ctx
         wfile = self._wfile
-        maxdiff = ctx._repo.maxdiff
+        maxdiff = ctx.repo().maxdiff
 
         self.error = None
         self.flabel = u'<b>%s</b>' % self.filePath()
@@ -539,7 +539,7 @@ class PatchDirData(_AbstractFileData):
             return
 
         if not force:
-            self.error = _checkdifferror(self.diff, ctx._repo.maxdiff)
+            self.error = _checkdifferror(self.diff, ctx.repo().maxdiff)
             if self.error:
                 self.error += u'\n\n' + forcedisplaymsg
 
@@ -609,7 +609,7 @@ class SubrepoData(_AbstractFileData):
                         logOutput += _ui.popbuffer()
                     return hglib.tounicode(logOutput)
 
-                opts = {'date':None, 'user':None, 'rev':[sfrom]}
+                opts = {'date':None, 'rev':[sfrom]}
                 subabspath = os.path.join(repo.root, subrelpath)
                 missingsub = srepo is None or not os.path.isdir(subabspath)
                 sfromlog = ''
@@ -702,7 +702,7 @@ class SubrepoData(_AbstractFileData):
 
             srev = ctx.substate.get(wfile, hglib.nullsubrepostate)[1]
             srepo = None
-            subabspath = os.path.join(ctx._repo.root, wfile)
+            subabspath = os.path.join(ctx.repo().root, wfile)
             sactual = ''
             if os.path.isdir(subabspath):
                 try:
@@ -748,14 +748,14 @@ class SubrepoData(_AbstractFileData):
                 sparent = ctx2.substate.get(wfile, hglib.nullsubrepostate)[1]
                 subrepochange, sstatedesc = \
                     genSubrepoRevChangedDescription(wfile,
-                        sparent, srev, ctx._repo)
+                        sparent, srev, ctx.repo())
                 out += subrepochange
             else:
                 sstatedesc = 'dirty'
                 if srev != sactual:
                     subrepochange, sstatedesc = \
                         genSubrepoRevChangedDescription(wfile,
-                            srev, sactual, ctx._repo)
+                            srev, sactual, ctx.repo())
                     out += subrepochange
                     if data:
                         sstatedesc += ' and dirty'
