@@ -7,8 +7,6 @@
 
 from __future__ import absolute_import
 
-import os
-
 from .qtcore import (
     QSettings,
     QTimer,
@@ -24,8 +22,6 @@ from .qtgui import (
     QVBoxLayout,
 )
 
-from mercurial import scmutil
-
 from ..util import hglib
 from ..util.i18n import _
 from . import (
@@ -39,11 +35,27 @@ from . import (
     wctxcleaner,
 )
 
+if hglib.TYPE_CHECKING:
+    from typing import (
+        Any,
+        List,
+        Optional,
+        Text,
+    )
+    from .qtgui import (
+        QWidget,
+    )
+    from .thgrepo import (
+        RepoAgent,
+    )
+
+
 BB = QDialogButtonBox
 
 class GraftDialog(QDialog):
 
     def __init__(self, repoagent, parent, **opts):
+        # type: (RepoAgent, Optional[QWidget], Any) -> None
         super(GraftDialog, self).__init__(parent)
         self.setWindowIcon(qtlib.geticon('hg-transplant'))
         self.setWindowFlags(self.windowFlags()
@@ -51,15 +63,14 @@ class GraftDialog(QDialog):
 
         self._repoagent = repoagent
         self._cmdsession = cmdcore.nullCmdSession()
-        self._graftstatefile = self.repo.vfs.join(b'graftstate')
         self.valid = True
 
         self.sourcelist = [
-            scmutil.revsymbol(self.repo, hglib.fromunicode(rev)).rev()
+            hglib.revsymbol(self.repo, hglib.fromunicode(rev)).rev()
             for rev in opts.get('source', ['.'])]
         currgraftrevs = self.graftstate()
         if currgraftrevs:
-            currgraftrevs = [scmutil.revsymbol(self.repo, rev).rev()
+            currgraftrevs = [hglib.revsymbol(self.repo, rev).rev()
                              for rev in currgraftrevs]
             if self.sourcelist != currgraftrevs:
                 res = qtlib.CustomPrompt(_('Interrupted graft operation found'),
@@ -155,6 +166,7 @@ class GraftDialog(QDialog):
         return self._repoagent.rawRepo()
 
     def _readSettings(self):
+        # type: () -> None
         qs = QSettings()
         qs.beginGroup('graft')
         for n, w in self._optchks.items():
@@ -167,6 +179,7 @@ class GraftDialog(QDialog):
         qs.endGroup()
 
     def _writeSettings(self):
+        # type: () -> None
         qs = QSettings()
         qs.beginGroup('graft')
         for n, w in self._optchks.items():
@@ -174,6 +187,7 @@ class GraftDialog(QDialog):
         qs.endGroup()
 
     def _updateSourceTitle(self, idx):
+        # type: (int) -> None
         numrevs = len(self.sourcelist)
         if numrevs <= 1:
             title = _('Graft changeset')
@@ -182,11 +196,13 @@ class GraftDialog(QDialog):
         self.srcb.setTitle(title)
 
     def _updateSource(self, idx):
+        # type: (int) -> None
         self._updateSourceTitle(idx)
         self.cslist.update(self.sourcelist[idx:])
 
     @pyqtSlot(bool)
     def _onCheckFinished(self, clean):
+        # type: (bool) -> None
         if not clean:
             self.graftbtn.setEnabled(False)
             txt = _('Before graft, you must '
@@ -199,12 +215,13 @@ class GraftDialog(QDialog):
         self._stbar.showMessage(txt)
 
     def graft(self):
+        # type: () -> None
         self.graftbtn.setEnabled(False)
         self.cancelbtn.setVisible(False)
         opts = dict((n, w.isChecked()) for n, w in self._optchks.items())
-        itool = opts.pop('autoresolve') and 'merge' or 'fail'
+        itool = 'merge' if opts.pop('autoresolve') else 'fail'
         opts['config'] = 'ui.merge=internal:%s' % itool
-        if os.path.exists(self._graftstatefile):
+        if hglib.graft_in_progress(self.repo):
             opts['continue'] = True
             args = []
         else:
@@ -214,15 +231,18 @@ class GraftDialog(QDialog):
         sess.commandFinished.connect(self._graftFinished)
 
     def abort(self):
+        # type: () -> None
         self.abortbtn.setDisabled(True)
         cmdline = hglib.buildcmdargs('graft', abort=True)
         sess = self._runCommand(cmdline)
         sess.commandFinished.connect(self._abortFinished)
 
     def graftstate(self):
+        # type: () -> Optional[List[bytes]]
         return hglib.readgraftstate(self.repo)
 
     def _runCommand(self, cmdline):
+        # type: (List[Text]) -> cmdcore.CmdSession
         assert self._cmdsession.isFinished()
         self._cmdsession = sess = self._repoagent.runCommand(cmdline, self)
         sess.commandFinished.connect(self._stbar.clearProgress)
@@ -233,6 +253,7 @@ class GraftDialog(QDialog):
 
     @pyqtSlot(int)
     def _graftFinished(self, ret):
+        # type: (int) -> None
         if self.checkResolve() is False:
             msg = _('Graft is complete')
             if ret == 255:
@@ -245,17 +266,20 @@ class GraftDialog(QDialog):
 
     @pyqtSlot()
     def _abortFinished(self):
+        # type: () -> None
         if self.checkResolve() is False:
             self._stbar.showMessage(_('Graft aborted'))
             self._makeCloseButton()
 
     def _makeCloseButton(self):
+        # type: () -> None
         self.graftbtn.setEnabled(True)
         self.graftbtn.setText(_('Close'))
         self.graftbtn.clicked.disconnect(self.graft)
         self.graftbtn.clicked.connect(self.accept)
 
     def checkResolve(self):
+        # type: () -> bool
         for root, path, status in thgrepo.recursiveMergeStatus(self.repo):
             if status == b'u':
                 txt = _('Graft generated merge <b>conflicts</b> that must '
@@ -287,6 +311,7 @@ class GraftDialog(QDialog):
             return False
 
     def linkActivated(self, cmd):
+        # type: (Text) -> None
         if cmd == 'resolve':
             dlg = resolve.ResolveDialog(self._repoagent, self)
             dlg.exec_()
@@ -295,9 +320,10 @@ class GraftDialog(QDialog):
             self._wctxcleaner.runCleaner(cmd)
 
     def reject(self):
+        # type: () -> None
         if self._wctxcleaner.isChecking():
             return
-        if os.path.exists(self._graftstatefile):
+        if hglib.graft_in_progress(self.repo):
             main = _('Exiting with an unfinished graft is not recommended.')
             text = _('Consider aborting the graft first.')
             labels = ((QMessageBox.Yes, _('&Exit')),
@@ -308,5 +334,6 @@ class GraftDialog(QDialog):
         super(GraftDialog, self).reject()
 
     def done(self, r):
+        # type: (int) -> None
         self._writeSettings()
         super(GraftDialog, self).done(r)
