@@ -35,6 +35,7 @@ from .qtcore import (
     pyqtSlot,
 )
 from .qtgui import (
+    QAction,
     QApplication,
     QComboBox,
     QCommonStyle,
@@ -64,6 +65,7 @@ from .qtgui import (
 
 from mercurial import (
     color,
+    encoding,
     extensions,
     pycompat,
     util,
@@ -89,8 +91,12 @@ except ImportError:
 
 if hglib.TYPE_CHECKING:
     from typing import (
+        Any,
         Dict,
+        List,
+        Text,
         Tuple,
+        Optional,
     )
 
 if pycompat.ispy3:
@@ -155,6 +161,7 @@ def openlocalurl(path):
     return QDesktopServices.openUrl(qurl)
 
 def editfiles(repo, files, lineno=None, search=None, parent=None):
+    # type: (Any, List[bytes], Optional[int], Optional[bytes], Optional[QWidget]) -> None
     if len(files) == 1:
         # if editing a single file, open in cwd context of that file
         filename = files[0].strip()
@@ -191,7 +198,7 @@ def editfiles(repo, files, lineno=None, search=None, parent=None):
             cmdline = cmdline.replace(b'$LINENUM', b'%d' % lineno)
         elif args:
             cmdline = b' '.join([toolpath, args])
-    elif lineno:
+    elif lineno is not None:
         if argsln:
             cmdline = b' '.join([toolpath, argsln])
             cmdline = cmdline.replace(b'$LINENUM', b'%d' % lineno)
@@ -738,7 +745,7 @@ def getcheckboxpixmap(state, bgcolor, widget):
 # icons to be rendered unusably small. The workaround for that is to render
 # the icons at double the normal size.
 # TODO: Remove this hack after upgrading to Qt5.
-IS_RETINA = stringutil.parsebool(os.environ.get('THG_RETINA', '0'))
+IS_RETINA = stringutil.parsebool(encoding.environ.get(b'THG_RETINA', b'0'))
 
 def _fixIconSizeForRetinaDisplay(s):
     if IS_RETINA:
@@ -872,10 +879,10 @@ class CustomPrompt(QMessageBox):
         super(CustomPrompt, self).keyPressEvent(event)
 
 class ChoicePrompt(QDialog):
-    def __init__(self, title, message, parent, choices, default=None,
-                 esc=None, files=None):
+    def __init__(self, title, message, parent, choices, default=None):
+        # type: (Text, Text, QWidget, List[Text], Optional[Text]) -> None
         QDialog.__init__(self, parent)
-        self.setWindowTitle(hglib.tounicode(title))
+        self.setWindowTitle(title)
         self.setWindowFlags(self.windowFlags() & ~Qt.WindowContextHelpButtonHint)
 
         self.box = QHBoxLayout()
@@ -888,7 +895,7 @@ class ChoicePrompt(QDialog):
 
         self.choice_combo = combo = QComboBox()
         self.choices = choices
-        combo.addItems([hglib.tounicode(item) for item in choices])
+        combo.addItems(choices)
         if default:
             try:
                 combo.setCurrentIndex(choices.index(default))
@@ -909,6 +916,7 @@ class ChoicePrompt(QDialog):
         self.setLayout(self.box)
 
     def run(self):
+        # type: () -> Optional[Text]
         if self.exec_():
             return self.choices[self.choice_combo.currentIndex()]
         return None
@@ -1390,6 +1398,7 @@ class Spacer(QWidget):
         return QSize(self.width, self.height)
 
 def getCurrentUsername(widget, repo, opts=None):
+    # type: (Optional[QWidget], Any, Optional[Dict[Text, Text]]) -> Optional[Text]
     if opts:
         # 1. Override has highest priority
         user = opts.get('user')
@@ -1399,7 +1408,7 @@ def getCurrentUsername(widget, repo, opts=None):
     # 2. Read from repository
     user = hglib.configuredusername(repo.ui)
     if user:
-        return user
+        return hglib.tounicode(user)
 
     # 3. Get a username from the user
     QMessageBox.information(widget, _('Please enter a username'),
@@ -1409,7 +1418,7 @@ def getCurrentUsername(widget, repo, opts=None):
     dlg = SettingsDialog(False, focus='ui.username')
     dlg.exec_()
     repo.invalidateui()
-    return hglib.configuredusername(repo.ui)
+    return hglib.tounicode(hglib.configuredusername(repo.ui))
 
 class _EncodingSafeInputDialog(QInputDialog):
     def accept(self):
@@ -1444,11 +1453,6 @@ def keysequence(o):
     except AttributeError:
         return QKeySequence(o)
 
-def modifiedkeysequence(o, modifier):
-    """Create QKeySequence of modifier key prepended"""
-    origseq = QKeySequence(keysequence(o))
-    return QKeySequence('%s+%s' % (modifier, origseq.toString()))
-
 def newshortcutsforstdkey(key, *args, **kwargs):
     """Create [QShortcut,...] for all key bindings of the given StandardKey"""
     return [QShortcut(keyseq, *args, **kwargs)
@@ -1501,3 +1505,10 @@ def setContextMenuShortcut(action, shortcut):
     action.setShortcut(shortcut)
     if QT_VERSION >= 0x50a00 and PYQT_VERSION >= 0x50a00:
         action.setShortcutVisibleInContextMenu(True)
+
+def setContextMenuShortcuts(action, shortcuts):
+    # type: (QAction, List[QKeySequence]) -> None
+    """Set shortcuts for a context menu action, making sure it's visible"""
+    action.setShortcuts(shortcuts)
+    if QT_VERSION >= 0x50a00 and PYQT_VERSION >= 0x50a00:
+        action.setShortcutVisibleInContextMenu(True)
