@@ -26,7 +26,6 @@ from .qtcore import (
     QRectF,
     QSettings,
     QSize,
-    QT_VERSION,
     Qt,
     pyqtSignal,
     pyqtSlot,
@@ -42,6 +41,7 @@ from .qtgui import (
     QDialogButtonBox,
     QFont,
     QFontMetrics,
+    QFormLayout,
     QHeaderView,
     QLabel,
     QListView,
@@ -53,18 +53,17 @@ from .qtgui import (
     QPen,
     QPolygonF,
     QProxyStyle,
+    QSpinBox,
     QStyle,
     QStyleOptionViewItemV2,
     QStyleOptionViewItemV4,
     QStyledItemDelegate,
     QTableView,
-    QVBoxLayout,
 )
 
 from mercurial import (
     error,
     pycompat,
-    scmutil,
 )
 
 from ..util import hglib
@@ -95,15 +94,12 @@ class HgRepoView(QTableView):
 
         vh = self.verticalHeader()
         vh.hide()
+        vh.setMinimumSectionSize(0)  # we'll set fixed size later
+        vh.setSectionResizeMode(QHeaderView.Fixed)
+
         header = self.horizontalHeader()
-        if QT_VERSION < 0x50000:
-            vh.setResizeMode(QHeaderView.Fixed)
-            header.setClickable(False)
-            header.setMovable(True)
-        else:
-            vh.setSectionResizeMode(QHeaderView.Fixed)
-            header.setSectionsClickable(False)
-            header.setSectionsMovable(True)
+        header.setSectionsClickable(False)
+        header.setSectionsMovable(True)
         header.setDefaultAlignment(Qt.AlignLeft)
         header.setHighlightSections(False)
         header.setContextMenuPolicy(Qt.CustomContextMenu)
@@ -135,7 +131,8 @@ class HgRepoView(QTableView):
         self.setSelectionBehavior(QAbstractItemView.SelectRows)
 
         self.doubleClicked.connect(self.revActivated)
-        self.clicked.connect(self.revClicked)
+        if repoagent.configBool('tortoisehg', 'copy_hash_selection'):
+            self.clicked.connect(self._copyHashToSelection)
 
     @property
     def repo(self):
@@ -170,9 +167,11 @@ class HgRepoView(QTableView):
     def setHistoryColumns(self):
         dlg = ColumnSelectDialog(self.colselect[1],
                                  self.model(), self.visibleColumns())
+        dlg.setRowHeight(self.defaultRowHeight())
         if dlg.exec_() == QDialog.Accepted:
             self.setVisibleColumns(dlg.selectedColumns())
             self.resizeColumns()
+            self.setDefaultRowHeight(dlg.rowHeight())
             self._saveColumnSettings()  # for new repository tab
 
     def _loadColumnSettings(self):
@@ -244,9 +243,24 @@ class HgRepoView(QTableView):
 
     def _adjustRowHeight(self):
         """Resize rows to fit to LabeledDelegate item"""
+        s = QSettings()
+        s.beginGroup(self.cfgname)
+        h = qtlib.readInt(s, 'row_height', default=-1)
+        s.endGroup()
+
+        if h < 0:
+            h = self._fallbackRowHeight()
+        self.setDefaultRowHeight(h)
+
+    def _fallbackRowHeight(self):
         fm = QFontMetrics(self.font())
-        h = fm.height() + 2 + 2  # see _LabelsLayout and LabeledDelegate
-        self.verticalHeader().setDefaultSectionSize(h)
+        return fm.height() + 2 + 2  # see _LabelsLayout and LabeledDelegate
+
+    def defaultRowHeight(self):
+        return self.verticalHeader().defaultSectionSize()
+
+    def setDefaultRowHeight(self, height):
+        self.verticalHeader().setDefaultSectionSize(height)
 
     @pyqtSlot()
     def _resizeIgnoreSettings(self):
@@ -308,7 +322,7 @@ class HgRepoView(QTableView):
     def context(self, rev):
         return self.repo[rev]
 
-    def revClicked(self, index):
+    def _copyHashToSelection(self, index):
         rev = self.revFromindex(index)
         if rev is not None:
             clip = QApplication.clipboard()
@@ -397,7 +411,7 @@ class HgRepoView(QTableView):
 
         try:
             # look for a mercurial commit hash
-            return scmutil.revsymbol(self.repo, symbolic_rev).rev()
+            return hglib.revsymbol(self.repo, symbolic_rev).rev()
         except error.RepoError:
             if b'hggit' not in self.repo.extensions():
                 # hg-git is not installed, do not try doing a gitnode() lookup
@@ -459,6 +473,13 @@ class HgRepoView(QTableView):
         if not s:
             s = QSettings()
 
+        s.beginGroup(self.cfgname)
+        if self.defaultRowHeight() == self._fallbackRowHeight():
+            s.remove('row_height')  # recalculate next time
+        else:
+            s.setValue('row_height', self.defaultRowHeight())
+        s.endGroup()
+
         model = self.model()
         assert model is not None
         col_widths = []
@@ -847,7 +868,7 @@ class ColumnSelectDialog(QDialog):
 
         disabled = [c for c in all if c not in curcolumns]
 
-        layout = QVBoxLayout()
+        layout = QFormLayout()
         layout.setContentsMargins(5, 5, 5, 5)
         self.setLayout(layout)
 
@@ -866,10 +887,15 @@ class ColumnSelectDialog(QDialog):
                 item.setCheckState(Qt.Unchecked)
             list.addItem(item)
         list.setDragDropMode(QListView.InternalMove)
-        layout.addWidget(list)
+        layout.addRow(list)
         self.list = list
 
-        layout.addWidget(QLabel(_('Drag to change order')))
+        layout.addRow(QLabel(_('Drag to change order')))
+
+        self._rowHeightEdit = edit = QSpinBox(self)
+        edit.setRange(10, 50)  # just limit to reasonable range
+        edit.setSuffix(_(' px'))
+        layout.addRow(_('Row height'), edit)
 
         # dialog buttons
         BB = QDialogButtonBox
@@ -886,3 +912,9 @@ class ColumnSelectDialog(QDialog):
                 # TODO: better to use data(role) instead
                 cols.append(item.columnid)  # pytype: disable=attribute-error
         return cols
+
+    def rowHeight(self):
+        return self._rowHeightEdit.value()
+
+    def setRowHeight(self, height):
+        self._rowHeightEdit.setValue(height)
