@@ -246,8 +246,7 @@ class RevDetailsWidget(QWidget, qtlib.TaskWidget):
         a.triggered.connect(self._applyFlatFileList)
 
         le = QLineEdit()
-        if hasattr(le, 'setPlaceholderText'): # Qt >= 4.7
-            le.setPlaceholderText(_('### filter text ###'))
+        le.setPlaceholderText(_('### filter text ###'))
         self.filefilter = le
         tbar.addWidget(self.filefilter)
         t = QTimer(self, interval=200, singleShot=True)
@@ -316,8 +315,8 @@ class RevDetailsWidget(QWidget, qtlib.TaskWidget):
         inlinetags = self._repoagent.configBool(
             'tortoisehg', 'issue.inlinetags')
         if ctx.tags() and inlinetags:
-            msg = ' '.join('[%s]' % hglib.tounicode(tag)
-                           for tag in ctx.tags()) + ' ' + msg
+            msg = b' '.join(b'[%s]' % tag
+                            for tag in ctx.tags()) + b' ' + msg
         # don't use <pre>...</pre>, which also changes font family
         self.message.setHtml('<div style="white-space: pre;">%s</div>'
                              % self._deschtmlize(msg))
@@ -372,7 +371,7 @@ class RevDetailsWidget(QWidget, qtlib.TaskWidget):
                 rev = b'tip'
         elif rev is not None:  # None in isrevsymbol() raises ProgrammingError
             if scmutil.isrevsymbol(self.repo, rev):
-                rev = scmutil.revsymbol(self.repo, rev).rev()
+                rev = hglib.revsymbol(self.repo, rev).rev()
             elif rev not in self.repo.thgmqunappliedpatches:
                 rev = b'tip'
         try:
@@ -629,8 +628,8 @@ class RevDetailsDialog(QDialog):
     @pyqtSlot(str)
     def linkActivated(self, link):
         def _torev(cset):
-            return scmutil.revsymbol(self._repoagent.rawRepo(),
-                                     hglib.fromunicode(cset))
+            return hglib.revsymbol(self._repoagent.rawRepo(),
+                                   hglib.fromunicode(cset))
 
         handlers = {'cset': lambda cset: self.setRev(_torev(cset)),
                     'repo': self._showRepoInWorkbench}
