@@ -191,7 +191,7 @@ class SignDialog(QDialog):
         user = qtlib.getCurrentUsername(self, self.repo)
         if not user:
             return
-        opts['user'] = hglib.tounicode(user)
+        opts['user'] = user
 
         cmdline = hglib.buildcmdargs('sign', self.rev, **opts)
         sess = self._repoagent.runCommand(cmdline, self)
