@@ -34,7 +34,6 @@ from .qtgui import (
     QFrame,
     QHBoxLayout,
     QKeySequence,
-    QLabel,
     QLineEdit,
     QMenu,
     QPushButton,
@@ -158,11 +157,7 @@ class StatusWidget(QWidget):
         tb.setIcon(qtlib.geticon('view-refresh'))
         tb.clicked.connect(self.refreshWctx)
         le = QLineEdit()
-        if hasattr(le, 'setPlaceholderText'): # Qt >= 4.7
-            le.setPlaceholderText(_('### filter text ###'))
-        else:
-            lbl = QLabel(_('Filter:'))
-            hbox.addWidget(lbl)
+        le.setPlaceholderText(_('### filter text ###'))
 
         st = ''
         for s in statusTypes:
@@ -1099,6 +1094,8 @@ class WctxModel(QAbstractTableModel):
                 self.rows.sort(key=lambda x: getStatusRank(x[col]))
             elif col == COL_MERGE_STATE:
                 self.rows.sort(key=lambda x: getMergeStatusRank(x[col]))
+            elif col == COL_SIZE:
+                self.rows.sort(key=lambda x: -1 if x[col] == '' else x[col])
             else:
                 self.rows.sort(key=lambda x: x[col])
 
