@@ -21,7 +21,7 @@ try:
         if val in ('1', 'True'):
             debugging += 'M'
         if debugging:
-            import win32traceutil
+            import win32traceutil  # pytype: disable=import-error
     except EnvironmentError:
         pass
 except (AttributeError, ImportError):    # AttributeError for winreg.* on Unix
